App.Util.Board = (function () {
  function Board(id) {
    this.id       = id;
    this.canvas   = document.getElementById(this.id);
    this.context  = this.canvas.getContext('2d');
    this.width    = this.canvas.width;
    this.height   = this.canvas.height;
    this.degrees  = Math.PI / 10.0;
    this.add      = Math.PI / 20.0;
    this.ring = {
      count  : 7,
      radius : (this.width > this.height) ? this.height / 14.5 : this.width / 14.5
    };
    this.center  = {
      x : this.width / 2,
      y : this.height / 2
    };
    this.color = {
      black  : '#222222',
      red    : '#CC3300',
      green  : '#5cb85c',
      white  : '#ffffff',
      gray   : '#080808',
      hover  : '#428bca',
      active : '#5bc0de'
    };
  }
  Board.prototype.reset = function () {
    this.context.beginPath();
    this.context.fillStyle   = 'rgba(0,0,0,0)';
    this.context.strokeStyle = 'rgba(0,0,0,0)';
    this.context.shadowColor = 'rgba(0,0,0,0)';
    this.context.shadowBlur  = 0;
  };
  Board.prototype.clear = function () {
    this.reset();
    this.context.clearRect(0, 0, this.width, this.height);
  };
  Board.prototype.arc = function (r, sa, sb, color) {
    var s, x, y;
    this.reset();
    this.context.moveTo(this.center.x, this.center.y);
    for (s = sa; s < sb; s += 1 / r) {
      x = r * Math.cos(s) + this.center.x;
      y = r * Math.sin(s) + this.center.y;
      this.context.lineTo(x, y);
    }
    this.context.moveTo(this.center.x, this.center.y);
    this.context.closePath();
    this.context.fillStyle = color;
    this.context.fill();
  };
  Board.prototype.spider = function () {
    var s, x, y;
    this.reset();
    this.context.moveTo(this.center.x, this.center.y);
    for (s = 0; s < Math.PI * 2; s += this.degrees) {
      x = this.ring.radius * this.ring.count * Math.cos(s + this.add) + this.center.x;
      y = this.ring.radius * this.ring.count * Math.sin(s + this.add) + this.center.y;
      this.context.moveTo(this.center.x, this.center.y);
      this.context.lineTo(x, y);
    }
    this.context.strokeStyle = this.color.gray;
    this.context.stroke();
  };
  Board.prototype.circle = function (ring, color) {
    var i, r, s, x, y;
    for (i = this.ring.count - 1; i >= 0; i -= 1) {
      r = this.ring.radius * (1 + i);
      this.reset();
      for (s = 0.0; s < 2 * Math.PI; s += 1 / r) {
        x = r * Math.cos(s) + this.center.x;
        y = r * Math.sin(s) + this.center.y;
        if (s === 0) {
          this.context.moveTo(x, y);
        } else {
          this.context.lineTo(x, y);
        }
      }
      this.context.closePath();
      if (i === 0) {
        this.context.fillStyle = (i === ring) ? color : this.color.black;
        this.context.fill();
      } else if (i === 1) {
        this.context.fillStyle = (i === ring) ? color : this.color.red;
        this.context.fill();
      }
      this.context.strokeStyle = this.color.gray;
      this.context.stroke();
    }
  };
  Board.prototype.text = function () {
    var i, s, x, y,
      scoreText = [6, 10, 15, 2, 17, 3, 19, 7, 16, 8, 11, 14, 9, 12, 5, 20, 1, 18, 4, 13];
    this.reset();
    this.context.moveTo(this.center.x, this.center.y);
    this.context.fillStyle    = "white";
    this.context.textBaseline = "middle";
    this.context.textAlign    = "center";
    this.context.font         = this.ring.radius - 16 + "px sans-serif";
    for (i = 0, s = 0; s < Math.PI * 2; s += this.degrees, i += 1) {
      x = this.ring.radius * (this.ring.count - 0.5) * Math.cos(s) + this.center.x;
      y = this.ring.radius * (this.ring.count - 0.5) * Math.sin(s) + this.center.y;
      this.context.fillText(scoreText[i], x, y);
    }
  };
  Board.prototype.draw = function () {
    var i,
        red     = this.color.red,
        green   = this.color.green,
        black   = this.color.black,
        white   = this.color.white,
        radius  = this.ring.radius,
        count   = this.ring.count,
        degrees = this.degrees,
        add     = this.add;
    this.clear();
    for (i = 0; i < 20; i += 1) {
      this.arc(radius * count, i * degrees + add, (i + 1) * degrees + add, black);
      if (i % 2 === 0) {
        this.arc(radius * (count - 1), i * degrees + add, (i + 1) * degrees + add, red);
        this.arc(radius * (count - 2), i * degrees + add, (i + 1) * degrees + add, black);
        this.arc(radius * (count - 3), i * degrees + add, (i + 1) * degrees + add, red);
        this.arc(radius * (count - 4), i * degrees + add, (i + 1) * degrees + add, black);
      } else {
        this.arc(radius * (count - 1), i * degrees + add, (i + 1) * degrees + add, green);
        this.arc(radius * (count - 2), i * degrees + add, (i + 1) * degrees + add, white);
        this.arc(radius * (count - 3), i * degrees + add, (i + 1) * degrees + add, green);
        this.arc(radius * (count - 4), i * degrees + add, (i + 1) * degrees + add, white);
      }
    }
    this.spider();
    this.circle();
    this.text();
  };
  Board.prototype.result = function (e) {
    // @TODO why can't use canvas.offset...
    // var x      = e.pageX - this.canvas.offsetLeft - this.center.x,
    //     y      = e.pageY - this.canvas.offsetTop  - this.center.y,

    // HiDPI support
    var x      = e.pageX - $('#' + this.id).offset().left - (this.center.x/2),
        y      = e.pageY - $('#' + this.id).offset().top  - (this.center.y/2),
        norm   = Math.sqrt(x * x + y * y),
        ux     = x / norm,
        uy     = y / norm,
        s      = Math.atan2(-ux, uy) + Math.PI / 2 - this.add,
        spider = Math.floor(s / this.degrees),
        ring   = Math.floor(norm / (this.ring.radius/2)), // HiDPI support
        points = [10, 15, 2, 17, 3, 19, 7, 16, 8, 11, 14, 9, 12, 5, 20, 1, 18, 4, 13, 6],
        point  = 0,
        label  = '';
    if (0 > spider) {spider += 20; }
    if (ring === 0) {
      point = 50;
      label = 'IN BULL';
      position = 'bull';
    } else if (ring === 1) {
      point = 25;
      label = 'OUT BULL';
      position = 'bull';
    } else if (ring === 3) {
      point = points[spider] * 3;
      label = 'TRIPLE ' + points[spider];
      position = 'triple';
    } else if (ring === 5) {
      point = points[spider] * 2;
      label = 'DOUBLE ' + points[spider];
      position = 'double';
    } else if (ring === 2 || ring === 4) {
      point = points[spider];
      label = 'SINGLE ' + points[spider];
      position = 'single';
    } else if (ring > 6) {
      point = 0;
      label = 'OUT';
      position = 'out';
    } else if (ring === 6) {
      return false;
    }
    return {
      label   : label,
      point   : point,
      position: position,
      area    : points[spider],
      ring    : ring,
      spider  : spider,
    };
  };
  Board.prototype.overwrite = function (e, color) {
    var i, temporary_color,
        result  = this.result(e),
        ring    = result.ring,
        spider  = result.spider,
        red     = this.color.red,
        black   = this.color.black,
        green   = this.color.green,
        white   = this.color.white,
        degrees = this.degrees,
        count   = this.ring.count,
        radius  = this.ring.radius,
        add     = this.add;
    this.clear();
    if (result.ring > 6) {
      this.context.fillStyle = color;
      this.context.fillRect(0, 0, this.width, this.height);
      this.reset();
    }
    for (i = 0; i < 20; i += 1) {
      this.arc(radius * count, i * degrees + add, (i + 1) * degrees + add, black);
      if (i % 2 === 0) {
        temporary_color = (ring + 1 === count - 1 && spider === i) ? color : red;
        this.arc(radius * (count - 1), i * degrees + add, (i + 1) * degrees + add, temporary_color);
        temporary_color = (ring + 1 === count - 2 && spider === i) ? color : black;
        this.arc(radius * (count - 2), i * degrees + add, (i + 1) * degrees + add, temporary_color);
        temporary_color = (ring + 1 === count - 3 && spider === i) ? color : red;
        this.arc(radius * (count - 3), i * degrees + add, (i + 1) * degrees + add, temporary_color);
        temporary_color = (ring + 1 === count - 4 && spider === i) ? color : black;
        this.arc(radius * (count - 4), i * degrees + add, (i + 1) * degrees + add, temporary_color);
      } else {
        temporary_color = (ring + 1 === count - 1 && spider === i) ? color : green;
        this.arc(radius * (count - 1), i * degrees + add, (i + 1) * degrees + add, temporary_color);
        temporary_color = (ring + 1 === count - 2 && spider === i) ? color : white;
        this.arc(radius * (count - 2), i * degrees + add, (i + 1) * degrees + add, temporary_color);
        temporary_color = (ring + 1 === count - 3 && spider === i) ? color : green;
        this.arc(radius * (count - 3), i * degrees + add, (i + 1) * degrees + add, temporary_color);
        temporary_color = (ring + 1 === count - 4 && spider === i) ? color : white;
        this.arc(radius * (count - 4), i * degrees + add, (i + 1) * degrees + add, temporary_color);
      }
    }
    this.spider();
    this.circle(ring, color);
    this.text();
  };
  return Board;
}());
